@@ -1,4 +1,4 @@
-/* $Id: xfburn-burn-audio-cd-composition-dialog.c 5905 2008-11-02 05:32:28Z squisher $ */
+/* $Id: xfburn-burn-audio-cd-composition-dialog.c 7718 2009-07-13 03:24:04Z squisher $ */
 /*
  *  Copyright (c) 2005-2007 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -57,9 +57,6 @@ typedef struct
   GtkWidget *entry;
   GtkWidget *check_eject;
   GtkWidget *check_burnfree;
-  GtkWidget *check_only_iso;
-  GtkWidget *hbox_iso;
-  GtkWidget *entry_path_iso;
   GtkWidget *check_dummy;
   GtkWidget *button_proceed;
 
@@ -79,19 +76,17 @@ static void xfburn_burn_audio_cd_composition_dialog_finalize (GObject * object);
 static void xfburn_burn_audio_cd_composition_dialog_get_property (GObject * object, guint prop_id, GValue * value, GParamSpec * pspec);
 static void xfburn_burn_audio_cd_composition_dialog_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec);
 
-static void cb_check_only_iso_toggled (GtkToggleButton * button, XfburnBurnAudioCdCompositionDialog * dialog);
-static void cb_browse_iso (GtkButton * button, XfburnBurnAudioCdCompositionDialog * dialog);
-static void cb_disc_refreshed (GtkWidget *device_box, XfburnDevice *device, XfburnBurnAudioCdCompositionDialog * dialog);
+static void cb_volume_changed (GtkWidget *device_box, gboolean device_changed, XfburnDevice *device, XfburnBurnAudioCdCompositionDialog * dialog);
 static void cb_dialog_response (XfburnBurnAudioCdCompositionDialog * dialog, gint response_id,
                                 XfburnBurnAudioCdCompositionDialogPrivate * priv);
 
 /* globals */
 static XfceTitledDialogClass *parent_class = NULL;
 
-GtkType
+GType
 xfburn_burn_audio_cd_composition_dialog_get_type ()
 {
-  static GtkType type = 0;
+  static GType type = 0;
 
   if (type == 0) {
     static const GTypeInfo our_info = {
@@ -140,13 +135,10 @@ xfburn_burn_audio_cd_composition_dialog_constructor (GType type, guint n_constru
   
   GdkPixbuf *icon = NULL;
   GtkBox *box;
-  GtkWidget *img;
   GtkWidget *frame;
   GtkWidget *vbox;
   GtkWidget *align;
   GtkWidget *button;
-  gchar *default_path;
-  gchar *tmp_dir;
   //const char *comp_name;
 
   gobj = G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_properties, construct_properties);
@@ -161,9 +153,8 @@ xfburn_burn_audio_cd_composition_dialog_constructor (GType type, guint n_constru
   g_object_unref (icon);
 
   /* burning devices list */
-  priv->device_box = xfburn_device_box_new (SHOW_CD_WRITERS | SHOW_CDRW_WRITERS | SHOW_SPEED_SELECTION);
-  g_signal_connect (G_OBJECT (priv->device_box), "disc-refreshed", G_CALLBACK (cb_disc_refreshed), obj);
-  g_signal_connect (G_OBJECT (priv->device_box), "device-changed", G_CALLBACK (cb_disc_refreshed), obj);
+  priv->device_box = xfburn_device_box_new (SHOW_CD_WRITERS | SHOW_CDRW_WRITERS | SHOW_SPEED_SELECTION | ACCEPT_ONLY_CD);
+  g_signal_connect (G_OBJECT (priv->device_box), "volume-changed", G_CALLBACK (cb_volume_changed), obj);
   gtk_widget_show (priv->device_box);
 
   priv->frame_device = xfce_create_framebox_with_content (_("Burning device"), priv->device_box);
@@ -218,39 +209,11 @@ xfburn_burn_audio_cd_composition_dialog_constructor (GType type, guint n_constru
   gtk_widget_show (priv->check_burnfree);
   gtk_box_pack_start (GTK_BOX (vbox), priv->check_burnfree, FALSE, FALSE, BORDER);
 
-  /* create ISO ? */
-  priv->check_only_iso = gtk_check_button_new_with_mnemonic (_("Only create _ISO"));
-  gtk_widget_show (priv->check_only_iso);
-  gtk_box_pack_start (GTK_BOX (vbox), priv->check_only_iso, FALSE, FALSE, BORDER);
-  g_signal_connect (G_OBJECT (priv->check_only_iso), "toggled", G_CALLBACK (cb_check_only_iso_toggled), obj);
-
   align = gtk_alignment_new (0, 0, 0, 0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (align), 0, 0, BORDER * 4, 0);
   gtk_widget_show (align);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
 
-  priv->hbox_iso = gtk_hbox_new (FALSE, 0);
-  gtk_widget_show (priv->hbox_iso);
-  gtk_container_add (GTK_CONTAINER (align), priv->hbox_iso);
-  gtk_widget_set_sensitive (priv->hbox_iso, FALSE);
-
-  priv->entry_path_iso = gtk_entry_new ();
-  tmp_dir = xfburn_settings_get_string ("temporary-dir", g_get_tmp_dir ());
-  default_path = g_build_filename (tmp_dir, "xfburn.iso", NULL);
-  gtk_entry_set_text (GTK_ENTRY (priv->entry_path_iso), default_path);
-  g_free (default_path);
-  g_free (tmp_dir);
-  gtk_widget_show (priv->entry_path_iso);
-  gtk_box_pack_start (GTK_BOX (priv->hbox_iso), priv->entry_path_iso, FALSE, FALSE, 0);
-
-  img = gtk_image_new_from_stock (GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_SMALL_TOOLBAR);
-  gtk_widget_show (img);
-  button = gtk_button_new ();
-  gtk_container_add (GTK_CONTAINER (button), img);
-  gtk_widget_show (button);
-  gtk_box_pack_start (GTK_BOX (priv->hbox_iso), button, FALSE, FALSE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (cb_browse_iso), obj);
-
   /* action buttons */
   button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   gtk_widget_show (button);
@@ -265,7 +228,7 @@ xfburn_burn_audio_cd_composition_dialog_constructor (GType type, guint n_constru
   gtk_widget_grab_focus (button);
   gtk_widget_grab_default (button);
 
-  cb_disc_refreshed (priv->device_box, xfburn_device_box_get_selected_device (XFBURN_DEVICE_BOX (priv->device_box)), obj);
+  cb_volume_changed (priv->device_box, TRUE, xfburn_device_box_get_selected_device (XFBURN_DEVICE_BOX (priv->device_box)), obj);
   g_signal_connect (G_OBJECT (obj), "response", G_CALLBACK (cb_dialog_response), priv);
 
   return gobj;
@@ -311,36 +274,7 @@ xfburn_burn_audio_cd_composition_dialog_finalize (GObject * object)
 
 /* internals */
 static void
-cb_check_only_iso_toggled (GtkToggleButton * button, XfburnBurnAudioCdCompositionDialog * dialog)
-{
-  XfburnBurnAudioCdCompositionDialogPrivate *priv = XFBURN_BURN_AUDIO_CD_COMPOSITION_DIALOG_GET_PRIVATE (dialog);
-  gboolean valid_disc;
-
-  gtk_widget_set_sensitive (priv->frame_device, !gtk_toggle_button_get_active (button));
-  xfburn_device_box_set_sensitive (XFBURN_DEVICE_BOX (priv->device_box), !gtk_toggle_button_get_active (button));
-  
-  gtk_widget_set_sensitive (priv->hbox_iso, gtk_toggle_button_get_active (button));
-  gtk_widget_set_sensitive (priv->check_eject, !gtk_toggle_button_get_active (button));
-  gtk_widget_set_sensitive (priv->check_burnfree, !gtk_toggle_button_get_active (button));
-  gtk_widget_set_sensitive (priv->check_dummy, !gtk_toggle_button_get_active (button));
-  if (!gtk_toggle_button_get_active (button)) {
-    g_object_get (G_OBJECT (priv->device_box), "valid", &valid_disc, NULL);
-    gtk_widget_set_sensitive (priv->button_proceed, valid_disc);
-  } else {
-    gtk_widget_set_sensitive (priv->button_proceed, TRUE);
-  }
-}
-
-static void
-cb_browse_iso (GtkButton * button, XfburnBurnAudioCdCompositionDialog * dialog)
-{
-  XfburnBurnAudioCdCompositionDialogPrivate *priv = XFBURN_BURN_AUDIO_CD_COMPOSITION_DIALOG_GET_PRIVATE (dialog);
-  
-  xfburn_browse_for_file (GTK_ENTRY (priv->entry_path_iso), GTK_WINDOW (dialog));
-}
-
-static void
-cb_disc_refreshed (GtkWidget *device_box, XfburnDevice *device, XfburnBurnAudioCdCompositionDialog * dialog)
+cb_volume_changed (GtkWidget *device_box, gboolean device_changed, XfburnDevice *device, XfburnBurnAudioCdCompositionDialog * dialog)
 {
   XfburnBurnAudioCdCompositionDialogPrivate *priv = XFBURN_BURN_AUDIO_CD_COMPOSITION_DIALOG_GET_PRIVATE (dialog);
   gboolean valid_disc;
@@ -350,83 +284,6 @@ cb_disc_refreshed (GtkWidget *device_box, XfburnDevice *device, XfburnBurnAudioC
   gtk_widget_set_sensitive (priv->button_proceed, valid_disc);
 }
 
-typedef struct {
-  GtkWidget *dialog_progress;
-  struct burn_source *src;
-  gchar *iso_path;
-} ThreadWriteIsoParams;
-
-static void
-thread_write_iso (ThreadWriteIsoParams * params)
-{
-  GtkWidget *dialog_progress = params->dialog_progress;
-  gint fd;
-  guchar buf[2048];
-  glong size = 0;
-  glong written = 0;
-  guint i = 0;
-  int (*read_fn) (struct burn_source *src, unsigned char *buf, int size);
-
-  fd = open (params->iso_path, O_WRONLY | O_CREAT | O_TRUNC, 0644);
-  if (fd == -1) {
-    /* could not create destination */
-    gchar err[256];
-    gchar *error_msg = NULL;
-
-    strerror_r (errno, err, 256);
-
-    error_msg = g_strdup_printf (_("Could not create destination ISO file: %s"), err);
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), error_msg);
-    g_free (error_msg);
-
-    goto end;
-  }
-
-  xfburn_progress_dialog_set_status_with_text (XFBURN_PROGRESS_DIALOG (dialog_progress), XFBURN_PROGRESS_DIALOG_STATUS_RUNNING, _("Writing ISO..."));
-
-  size = (glong) params->src->get_size (params->src);
-  if (params->src->read == NULL)
-    read_fn = params->src->read_xt;
-  else
-    read_fn = params->src->read;
-
-  /* FIXME: is size really always 2048? */
-  while (read_fn (params->src, buf, 2048) == 2048) {
-    if (write (fd, buf, 2048) < 2048) {
-      /* an error occured while writing */
-      gchar err[256];
-      gchar *error_msg = NULL;
-      
-      strerror_r (errno, err, 256);
-    
-      error_msg = g_strdup_printf (_("An error occured while writing ISO: %s"), err);
-      xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), error_msg);
-      g_free (error_msg);
-      goto cleanup;
-    } else {
-      written += 2048;
-      i++;
-
-      if (i >= 1000) {
-	i = 0;
-	gdouble percent = 0;
-
-	percent = ((gdouble) written / (gdouble) size);
-
-	xfburn_progress_dialog_set_progress_bar_fraction (XFBURN_PROGRESS_DIALOG (dialog_progress), percent);
-      }
-    }
-  }
-  xfburn_progress_dialog_set_status_with_text (XFBURN_PROGRESS_DIALOG (dialog_progress), XFBURN_PROGRESS_DIALOG_STATUS_COMPLETED, _("Done"));
-
- cleanup:
-  close (fd);
- end:
-  burn_source_free (params->src);
-  g_free (params->iso_path);
-  g_free (params);
-}
-
 typedef struct {
   GtkWidget *dialog_progress;
   XfburnDevice *device;
@@ -450,7 +307,8 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
 
   ret = burn_disc_add_session (disc, session, BURN_POS_END);
   if (ret == 0) {
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to create disc object"));
+    g_warning ("Unable to create disc object");
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("A problem with the burn backend occurred."));
     return;
   }
 
@@ -482,7 +340,7 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
     burn_write_opts_set_write_type (burn_options, BURN_WRITE_RAW, BURN_BLOCK_RAW96R);
     break;
   default:
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("The write mode is not supported currently"));
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("The write mode is not supported currently."));
     return;
   }
 
@@ -492,9 +350,7 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
   burn_write_opts_set_underrun_proof (burn_options, params->burnfree ? 1 : 0);
   burn_drive_set_buffer_waiting (drive, 1, -1, -1, -1, 75, 95);
 
-  // FIXME: perform_burn_write only takes one fifo as an argument right now
-  //xfburn_perform_burn_write (dialog_progress, drive, params->write_mode, burn_options, disc, srcs, track_sectors);
-  xfburn_perform_burn_write (dialog_progress, drive, params->write_mode, burn_options, disc, NULL, track_sectors);
+  xfburn_perform_burn_write (dialog_progress, drive, params->write_mode, burn_options, AUDIO_BYTES_PER_SECTOR, disc, srcs, track_sectors);
 
   burn_write_opts_free (burn_options);
 }
@@ -518,13 +374,6 @@ thread_burn_composition (ThreadBurnCompositionParams * params)
 
   struct burn_drive_info *drive_info = NULL;
 
-  if (!burn_initialize ()) {
-    g_critical ("Unable to initialize libburn");
-    /* FIXME: free contents of params! */
-    g_free (params);
-    return;
-  }
-
   disc = burn_disc_create ();
   session = burn_session_create ();
 
@@ -562,10 +411,10 @@ thread_burn_composition (ThreadBurnCompositionParams * params)
 
   if (!abort) {
     if (!xfburn_device_grab (params->device, &drive_info)) {
-      xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to grab drive"));
+      xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to grab the drive."));
     } else {
       thread_burn_prep_and_burn (params, drive_info->drive, disc, session, n_tracks, track_sectors, tracks, srcs);
-      burn_drive_release (drive_info->drive, params->eject ? 1 : 0);
+      xfburn_device_release (drive_info, params->eject);
     }
   }
 
@@ -578,13 +427,10 @@ thread_burn_composition (ThreadBurnCompositionParams * params)
   g_free (tracks);
   g_free (track_sectors);
 
-  for (track_list = params->tracks; track_list; track_list = g_slist_next (track_list))
-    xfburn_transcoder_free_burning_resources (trans, (XfburnAudioTrack *) track_list->data, NULL);
   g_object_unref (trans);
 
   burn_session_free (session);
   burn_disc_free (disc);
-  burn_finish ();
 
   /* FIXME: free track_list here? */
   g_free (params);
@@ -596,8 +442,6 @@ cb_dialog_response (XfburnBurnAudioCdCompositionDialog * dialog, gint response_i
   if (response_id == GTK_RESPONSE_OK) {
     GtkWidget *dialog_progress;
 
-    struct burn_source * src = NULL;
-
     /* If the name was the default, update the image volume id and volset id */
     /*
     if (priv->entry != NULL) {
@@ -613,40 +457,28 @@ cb_dialog_response (XfburnBurnAudioCdCompositionDialog * dialog, gint response_i
     
     gtk_widget_show (dialog_progress);
 
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_only_iso))) {
-      ThreadWriteIsoParams *params = NULL;
+    ThreadBurnCompositionParams *params = NULL;
+    XfburnDevice *device;
+    gint speed;
+    XfburnWriteMode write_mode;
+    //struct burn_source * src_fifo = NULL;
 
-      /* create a new iso */
-      params = g_new0 (ThreadWriteIsoParams, 1);
-      params->dialog_progress = dialog_progress;
-      params->src = src;
-      params->iso_path = g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->entry_path_iso)));
-      g_thread_create ((GThreadFunc) thread_write_iso, params, FALSE, NULL);
-    }
-    else {
-      ThreadBurnCompositionParams *params = NULL;
-      XfburnDevice *device;
-      gint speed;
-      XfburnWriteMode write_mode;
-      //struct burn_source * src_fifo = NULL;
+    device = xfburn_device_box_get_selected_device (XFBURN_DEVICE_BOX (priv->device_box));
+    speed = xfburn_device_box_get_speed (XFBURN_DEVICE_BOX (priv->device_box));
+    /* cdrskin burns audio with SAO */
+    write_mode = WRITE_MODE_SAO;
 
-      device = xfburn_device_box_get_selected_device (XFBURN_DEVICE_BOX (priv->device_box));
-      speed = xfburn_device_box_get_speed (XFBURN_DEVICE_BOX (priv->device_box));
-      /* cdrskin burns audio with SAO */
-      write_mode = WRITE_MODE_SAO;
-
-      /* burn composition */
-      params = g_new0 (ThreadBurnCompositionParams, 1);
-      params->dialog_progress = dialog_progress;
-      params->device = device;
-      params->tracks = priv->track_list;
-      params->speed = speed;
-      params->write_mode = write_mode;
-      params->eject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_eject));
-      params->dummy = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_dummy));
-      params->burnfree = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_burnfree));
-      g_thread_create ((GThreadFunc) thread_burn_composition, params, FALSE, NULL);
-    }
+    /* burn composition */
+    params = g_new0 (ThreadBurnCompositionParams, 1);
+    params->dialog_progress = dialog_progress;
+    params->device = device;
+    params->tracks = priv->track_list;
+    params->speed = speed;
+    params->write_mode = write_mode;
+    params->eject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_eject));
+    params->dummy = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_dummy));
+    params->burnfree = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_burnfree));
+    g_thread_create ((GThreadFunc) thread_burn_composition, params, FALSE, NULL);
   }
 }
 
