@@ -1,4 +1,4 @@
-/* $Id: xfburn-data-composition.c 6125 2008-11-16 19:40:19Z squisher $ */
+/* $Id: xfburn-data-composition.c 7732 2009-07-16 14:41:23Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -32,6 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <time.h>
 
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
@@ -169,6 +170,7 @@ typedef struct
   GtkWidget *content;
   GtkWidget *disc_usage;
   GtkWidget *progress;
+  GtkTreeStore *model;
 
 } XfburnDataCompositionPrivate;
 
@@ -203,10 +205,10 @@ static GdkPixbuf *icon_directory = NULL, *icon_file = NULL;
 /***************************/
 /* XfburnDataComposition class */
 /***************************/
-GtkType
+GType
 xfburn_data_composition_get_type (void)
 {
-  static GtkType data_composition_type = 0;
+  static GType data_composition_type = 0;
 
   if (!data_composition_type) {
     static const GTypeInfo data_composition_info = {
@@ -282,10 +284,19 @@ xfburn_data_composition_init (XfburnDataComposition * composition)
 
   GtkTargetEntry gte_src[] =  { { "XFBURN_TREE_PATHS", GTK_TARGET_SAME_WIDGET, DATA_COMPOSITION_DND_TARGET_INSIDE } };
   GtkTargetEntry gte_dest[] = { { "XFBURN_TREE_PATHS", GTK_TARGET_SAME_WIDGET, DATA_COMPOSITION_DND_TARGET_INSIDE },
-                                { "text/plain", 0, DATA_COMPOSITION_DND_TARGET_TEXT_PLAIN },
+#ifdef HAVE_THUNAR_VFS
                                 { "text/uri-list", 0, DATA_COMPOSITION_DND_TARGET_TEXT_URI_LIST },
+#endif
+                                { "text/plain;charset=utf-8", 0, DATA_COMPOSITION_DND_TARGET_TEXT_PLAIN },
                               };
 
+  gchar *vol_name;
+  char timestr[80];
+  struct tm *today;
+  time_t tm;
+  /* FIXME: put i into the class? */
+  static int i = 0;
+
   priv->full_paths_to_add = NULL;
 
   instances++;
@@ -345,7 +356,18 @@ xfburn_data_composition_init (XfburnDataComposition * composition)
   gtk_widget_show (label);
   
   priv->entry_volume_name = gtk_entry_new ();
-  gtk_entry_set_text (GTK_ENTRY (priv->entry_volume_name), _(DATA_COMPOSITION_DEFAULT_NAME));
+
+  tm = time (NULL);
+  today = localtime (&tm);
+
+  if (tm && strftime (timestr, 80, "%Y-%m-%d", today))
+    /* Note to translators: first %s is the date in "i18n" format (year-month-day), %d is a running number of compositions */
+    vol_name = g_strdup_printf (_("Data %s~%d"), timestr, ++i);
+  else
+    vol_name = g_strdup_printf ("%s %d", _(DATA_COMPOSITION_DEFAULT_NAME), ++i);
+
+  gtk_entry_set_text (GTK_ENTRY (priv->entry_volume_name), vol_name);
+  g_free (vol_name);
   gtk_box_pack_start (GTK_BOX (hbox), priv->entry_volume_name, FALSE, FALSE, 0);
   gtk_widget_show (priv->entry_volume_name);
   
@@ -359,6 +381,7 @@ xfburn_data_composition_init (XfburnDataComposition * composition)
   priv->content = exo_tree_view_new ();
   model = gtk_tree_store_new (DATA_COMPOSITION_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
                               G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_UINT);
+  priv->model = model;
 							  
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), DATA_COMPOSITION_COLUMN_CONTENT,
                                    directory_tree_sortfunc, NULL, NULL);
@@ -452,6 +475,8 @@ xfburn_data_composition_finalize (GObject * object)
       icon_file = NULL;
     }
   }
+
+  g_object_unref (priv->model);
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1011,27 +1036,30 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
   struct stat s;
 
   if ((g_lstat (path, &s) == 0)) {
-    //gchar *basename = NULL;
     gchar *humansize = NULL;
     GtkTreeIter *parent = NULL;
     GtkTreePath *tree_path = NULL;
+    int parent_type = 0;
 
     if (!(S_ISDIR (s.st_mode) ||S_ISREG (s.st_mode) || S_ISCHR(s.st_mode) || S_ISBLK(s.st_mode) || S_ISLNK (s.st_mode))) {
       return FALSE;
     }
     
-    /* allow hidden files
-    basename = g_path_get_basename (path);
-    if ( (strlen (basename) > 1) && (basename[0] == '.') ) {
-
-      g_free (basename);  
-      return FALSE;
-    }
-    g_free (basename);
-    */
-    
     xfburn_adding_progress_pulse (XFBURN_ADDING_PROGRESS (priv->progress));
     
+    /* ensure that we can only drop on top of folders, not files */
+    if (insertion) {
+      gtk_tree_model_get (model, insertion, DATA_COMPOSITION_COLUMN_TYPE, &parent_type, -1);
+
+      if (parent_type == DATA_COMPOSITION_TYPE_FILE) {
+        DBG ("Parent is file, and we're dropping into %d", position);
+        if (position == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
+          position = GTK_TREE_VIEW_DROP_AFTER;
+        else if (position == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
+          position = GTK_TREE_VIEW_DROP_BEFORE;
+      }      
+    }
+
     /* find parent */
     switch (position){
       case GTK_TREE_VIEW_DROP_BEFORE:
@@ -1142,6 +1170,14 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
       const gchar *mime_icon_name = NULL;
       GdkPixbuf *mime_icon = NULL;
       gint x,y;
+
+      if (s.st_size > MAXIMUM_ISO_FILE_SIZE) {
+        gdk_threads_enter ();
+        xfce_err (_("%s cannot be added to the composition, because it exceeds the maximum allowed file size for iso9660."), path);
+        gdk_threads_leave ();
+
+        return FALSE;
+      }
 	  
       gdk_threads_enter ();
       screen = gtk_widget_get_screen (GTK_WIDGET (dc));
@@ -1586,32 +1622,59 @@ cb_content_drag_data_rcv (GtkWidget * widget, GdkDragContext * dc, guint x, guin
     gtk_widget_hide (priv->progress);
     xfburn_default_cursor (priv->content);
   }
-  /* drag from the file selector */
-  else if (sd->target == gdk_atom_intern ("text/plain", FALSE)) {
+  /* drag from the file selector, or nautilus */
+  else if (sd->target == gdk_atom_intern ("text/plain;charset=utf-8", FALSE)) {
     ThreadAddFilesDragParams *params;
     const gchar *file = NULL;
+    gchar *full_paths;
 
     xfburn_adding_progress_show (XFBURN_ADDING_PROGRESS (priv->progress));
+
+    full_paths = (gchar *) gtk_selection_data_get_text (sd);
     
-    file = strtok ((gchar *) sd->data, "\n");
+    file = strtok ((gchar *) full_paths, "\n");
     while (file) {
       gchar *full_path;
 
+#ifdef HAVE_THUNAR_VFS
+      ThunarVfsPath *vfs_path;
+      GError *vfs_error = NULL;
+
+      vfs_path = thunar_vfs_path_new (file, &vfs_error);
+
+      if (vfs_error) {
+        g_warning ("Failed to create vfs path for '%s': %s", file, vfs_error->message);
+        g_error_free (vfs_error);
+        continue;
+      }
+
+      if (thunar_vfs_path_get_scheme (vfs_path) != THUNAR_VFS_PATH_SCHEME_FILE)
+        continue;
+      full_path = thunar_vfs_path_dup_string (vfs_path);
+
+      thunar_vfs_path_unref (vfs_path);
+
+#else /* no thunar-vfs */
+
       if (g_str_has_prefix (file, "file://"))
         full_path = g_build_filename (&file[7], NULL);
-      else if (g_str_has_prefix ((gchar *) sd->data, "file:"))
+      else if (g_str_has_prefix (file, "file:"))
         full_path = g_build_filename (&file[5], NULL);
       else
         full_path = g_build_filename (file, NULL);
 
       if (full_path[strlen (full_path) - 1] == '\r')
         full_path[strlen (full_path) - 1] = '\0';
+#endif
+
+      DBG ("Adding path '%s'", full_path);
 
       /* remember path to add it later in another thread */
       priv->full_paths_to_add = g_list_append (priv->full_paths_to_add, full_path);
 
       file = strtok (NULL, "\n");
     }
+    g_free (full_paths);
 
     priv->full_paths_to_add = g_list_reverse (priv->full_paths_to_add);
     priv->path_where_insert = path_where_insert;
@@ -1742,11 +1805,24 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
       IsoNode *node = NULL;
       IsoDir *dir = NULL;
       int r;
+      gchar *basename;
       
       gtk_tree_model_get (model, iter, DATA_COMPOSITION_COLUMN_TYPE, &type,
 			  DATA_COMPOSITION_COLUMN_CONTENT, &name, DATA_COMPOSITION_COLUMN_PATH, &src, -1);
 
-      r = iso_tree_add_node (image, parent, src, &node);
+      if (type == DATA_COMPOSITION_TYPE_DIRECTORY && src == NULL) {
+        /* a directory which the user added for this composition */
+        r = iso_tree_add_new_dir (parent, name, &dir);
+        node = (IsoNode *) dir;
+
+        /* if the new directory is part of the root of the iso,
+         * then its owner will be set to root:root. Not sure what a better
+         * default could be, so I'll just leave it like that. */
+      } else {
+        /* something existing on the filesystem, creating a node
+         * will copy its attributes */
+        r = iso_tree_add_node (image, parent, src, &node);
+      }
 
       if (r < 0) {
         if (r == ISO_NULL_POINTER)
@@ -1759,12 +1835,26 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
           g_error ("Failed adding %s as a node to the image: code %d!", src, r);
       }
 
-      /* why was this called again? iso_tree_add_node seems to do
-       * a good job setting the name by itself.
-      if (type != DATA_COMPOSITION_TYPE_DIRECTORY)
-        iso_node_set_name (node, name);
-      */
+      basename = g_path_get_basename (src);
 
+      /* check if the file has been renamed */
+      if (strcmp (basename, name) != 0) {
+        /* rename the iso_node */
+        r = iso_node_set_name (node, name);
+
+        if (r == 0) {
+          /* The first string is the renamed name, the second one the original name */
+          xfce_warn (_("Duplicate filename '%s' for '%s'"), name, src);
+
+          g_free (basename);
+          g_free (name);
+          g_free (src);
+
+          continue;
+        }
+      }
+
+      g_free (basename);
       g_free (name);
       g_free (src);
 
@@ -1773,6 +1863,7 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
 
         if (iso_node_get_type(node) != LIBISO_DIR)
             g_error ("Expected %s to be a directory, but it isn't...\n", src);
+
         dir = (IsoDir *)node;
 
 	gtk_tree_model_iter_children (model, &child, iter);
