@@ -1,4 +1,4 @@
-/* $Id: xfburn-disc-usage.c 6045 2008-11-10 03:48:57Z squisher $ */
+/* $Id: xfburn-disc-usage.c 7506 2009-06-09 01:47:30Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  * Copyright (c) 2008      David Mohr <david@mcbf.net>
@@ -89,10 +89,10 @@ static guint signals[LAST_SIGNAL];
 /*******************************/
 static guint signals[LAST_SIGNAL];
 
-GtkType
+GType
 xfburn_disc_usage_get_type (void)
 {
-  static GtkType disc_usage_type = 0;
+  static GType disc_usage_type = 0;
 
   if (!disc_usage_type) {
     static const GTypeInfo disc_usage_info = {
