@@ -1,4 +1,4 @@
-/* $Id: xfburn-main.c 5782 2008-10-23 23:49:19Z squisher $ */
+/* $Id: xfburn-main.c 7695 2009-07-08 14:23:15Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -57,6 +57,10 @@
 static gboolean parse_option (const gchar *option_name, const gchar *value,
 			      gpointer data, GError **error);
 
+/* globals */
+static int window_counter = 0;
+
+
 /* command line parameters */
 static gchar *image_filename = NULL;
 static gboolean show_version = FALSE;
@@ -66,18 +70,26 @@ static gboolean add_data_composition = FALSE;
 static gboolean add_audio_composition = FALSE;
 static gboolean blank = FALSE;
 static gchar *transcoder_selection = NULL;
+#if 0 /* INITIAL_DIRECTORY_OPTION */
+static gchar *initial_dir = NULL;
+#endif
 
 static GOptionEntry optionentries[] = {
   { "burn-image", 'i', G_OPTION_FLAG_OPTIONAL_ARG /* || G_OPTION_FLAG_FILENAME */, G_OPTION_ARG_CALLBACK, &parse_option, 
-    "Open the burn image dialog. The filename of the image can optionally be specified as a parameter", NULL },
+    "Open the burn image dialog, optionally followed by the image filename", NULL },
   { "blank", 'b', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &parse_option, 
-    "Open the blank disc dialog.", NULL },
+    "Open the blank disc dialog", NULL },
   { "data-composition", 'd', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &parse_option, 
-    "Start a data composition. Optionally followed by files/directories to be added to the composition.", NULL },
+    "Start a data composition, optionally followed by files/directories to be added to the composition", NULL },
   { "audio-composition", 'a', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &parse_option, 
-    "Start an audio composition. Optionally followed by files/directories to be added to the composition.", NULL },
+    "Start an audio composition, optionally followed by files/directories to be added to the composition", NULL },
   { "transcoder", 't', 0, G_OPTION_ARG_STRING, &transcoder_selection, 
-    "Select the transcoder. Run with --transcoder=list to see the available ones.", NULL },
+    "Select the transcoder, run with --transcoder=list to see the available ones", NULL },
+#if 0 /* INITIAL_DIRECTORY_OPTION */
+/* Implementing this does not seem worth the effort */
+  { "directory", 'D', G_OPTION_FLAG_OPTIONAL_ARG , G_OPTION_ARG_CALLBACK, &parse_option, 
+    "Start the file browser in the specified directory, or the current directory if none is specified (the default is to start in your home directory)", NULL },
+#endif
   { "version", 'V', G_OPTION_FLAG_NO_ARG , G_OPTION_ARG_NONE, &show_version, 
     "Display program version and exit", NULL },
   { "main", 'm', G_OPTION_FLAG_NO_ARG , G_OPTION_ARG_NONE, &show_main, 
@@ -85,31 +97,8 @@ static GOptionEntry optionentries[] = {
   { NULL, ' ', 0, 0, NULL, NULL, NULL }
 };
 
-static gboolean parse_option (const gchar *option_name, const gchar *value,
-                              gpointer data, GError **error)
-{
-  if (strcmp (option_name, "-i") == 0 || strcmp (option_name, "--burn-image") == 0) {
-    if (value == NULL)
-      image_filename = "";
-    else
-      image_filename = g_strdup(value);
-  } else if (strcmp (option_name, "-d") == 0 || strcmp (option_name, "--data-composition") == 0) {
-    add_data_composition = TRUE;
-  } else if (strcmp (option_name, "-a") == 0 || strcmp (option_name, "--audio-composition") == 0) {
-    add_audio_composition = TRUE;
-  } else if (strcmp (option_name, "-b") == 0 || strcmp (option_name, "--blank") == 0) {
-    blank = TRUE;
-  } else {
-    g_set_error (error, 0, G_OPTION_ERROR_FAILED, "Invalid command line option. Please report, this is a bug.");
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-/* globals */
-static int window_counter = 0;
 
+/* public functions */
 void
 xfburn_main_enter_window ()
 {
@@ -137,6 +126,49 @@ xfburn_main_enter_main_window ()
   window_counter = -42;
 }
 
+#if 0 /* INITIAL_DIRECTORY_OPTION */
+const gchar *
+xfburn_main_get_initial_dir ()
+{
+  if (initial_dir)
+    return initial_dir;
+  else
+    return xfce_get_homedir ();
+}
+#endif
+
+
+/* private functions */
+
+static gboolean parse_option (const gchar *option_name, const gchar *value,
+                              gpointer data, GError **error)
+{
+  if (strcmp (option_name, "-i") == 0 || strcmp (option_name, "--burn-image") == 0) {
+    if (value == NULL)
+      image_filename = "";
+    else
+      image_filename = g_strdup(value);
+  } else if (strcmp (option_name, "-d") == 0 || strcmp (option_name, "--data-composition") == 0) {
+    add_data_composition = TRUE;
+  } else if (strcmp (option_name, "-a") == 0 || strcmp (option_name, "--audio-composition") == 0) {
+    add_audio_composition = TRUE;
+  } else if (strcmp (option_name, "-b") == 0 || strcmp (option_name, "--blank") == 0) {
+    blank = TRUE;
+#if 0 /* INITIAL_DIRECTORY_OPTION */
+  } else if (strcmp (option_name, "-D") == 0 || strcmp (option_name, "--directory") == 0) {
+    if (value == NULL)
+      initial_dir = g_get_current_dir ();
+    else
+      initial_dir = g_strdup(value);
+#endif
+  } else {
+    g_set_error (error, 0, G_OPTION_ERROR_FAILED, "Invalid command line option. Please report, this is a bug.");
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 
 void
 print_available_transcoders ()
@@ -153,12 +185,13 @@ int
 main (int argc, char **argv)
 {
   GtkWidget *mainwin;
-  gint n_drives;
+  gint n_burners;
   GError *error = NULL;
 #ifdef HAVE_HAL
   gchar *error_msg;
 #endif
   XfburnTranscoder *transcoder;
+  XfburnDeviceList *devlist;
 
 #if DEBUG > 0
   /* I have to disable this until GtkTreeView gets fixed,
@@ -184,10 +217,19 @@ main (int argc, char **argv)
     }
   }
 
+  if (!burn_initialize ()) {
+    g_critical ("Unable to initialize libburn");
+    xfce_err (_("Unable to initialize the burning backend."));
+    gdk_threads_leave ();
+    return EXIT_FAILURE;
+  }
+
 #ifdef HAVE_GST
   if (!gst_init_check (&argc, &argv, &error)) {
-    g_warning ("Failed to initialize gstreamer!");
-    /* later make this a soft failure, and just disable gstreamer functionality */
+    g_critical ("Failed to initialize gstreamer!");
+    /* I'm assuming this pretty much never happens. If it does, we should make this a soft failure and fall back to basic */
+    gdk_threads_leave ();
+    burn_finish ();
     return EXIT_FAILURE;
   }
 #endif
@@ -222,9 +264,12 @@ main (int argc, char **argv)
 
   if (transcoder_selection && strcmp (transcoder_selection, "list") == 0) {
     print_available_transcoders();
+    gdk_threads_leave ();
+    burn_finish ();
     return EXIT_SUCCESS;
   }
 
+  DBG ("%s version %s for Xfce %s\n", PACKAGE, VERSION, xfce_version_string ());
 
   xfburn_settings_init ();
   
@@ -243,79 +288,34 @@ main (int argc, char **argv)
     thunar_vfs_shutdown ();
 #endif
     gdk_threads_leave ();
+    burn_finish ();
     return EXIT_FAILURE;
+  } else {
+    g_message ("Using HAL");
   }
 #endif
 
   xfburn_stock_init ();
-  n_drives = xfburn_device_list_init ();
-#if 0
-#ifdef HAVE_THUNAR_VFS
-  if (n_drives < 1) {
-    //XfburnHalManager *halman = xfburn_hal_manager_get_instance();
-    ThunarVfsVolumeManager *volman;
-    GList *volume_list;
-    gboolean unmounted = FALSE;
-    gchar *mp_name;
+  devlist = xfburn_device_list_new ();
+  g_object_get (devlist, "num-burners", &n_burners, NULL);
 
-    volman = thunar_vfs_volume_manager_get_default();
-    volume_list = thunar_vfs_volume_manager_get_volumes (volman);
-
-    while (volume_list) {
-      ThunarVfsVolume *vol = THUNAR_VFS_VOLUME (volume_list->data);
-      ThunarVfsPath *mp;
-
-      switch (thunar_vfs_volume_get_kind (vol)) {
-        case THUNAR_VFS_VOLUME_KIND_CDROM:
-        case THUNAR_VFS_VOLUME_KIND_CDR:
-        case THUNAR_VFS_VOLUME_KIND_DVDROM:
-        case THUNAR_VFS_VOLUME_KIND_DVDRAM:
-        case THUNAR_VFS_VOLUME_KIND_DVDR:
-        case THUNAR_VFS_VOLUME_KIND_DVDRW:
-        case THUNAR_VFS_VOLUME_KIND_DVDPLUSR:
-        case THUNAR_VFS_VOLUME_KIND_DVDPLUSRW:
-        case THUNAR_VFS_VOLUME_KIND_AUDIO_CD:
-          if (thunar_vfs_volume_is_mounted (vol)) {
-            mp = thunar_vfs_volume_get_mount_point (vol);
-            mp_name = thunar_vfs_path_dup_string (mp);
-            if (thunar_vfs_volume_unmount (vol, NULL, NULL)) {
-              unmounted = TRUE;
-              g_message ("Unmounted drive %s", mp_name);
-            } else {
-              g_message ("Failed to unmounted drive %s", mp_name);
-            }
-            g_free (mp_name);
-          }
-          break;
-        default:
-          break;
-      }
-
-      volume_list = g_list_next (volume_list);
-    }
-
-    g_object_unref (volman);
-
-    if (unmounted)
-      n_drives = xfburn_device_list_init ();
-    else
-      g_debug ("Could not umount any optical drives.");
-  }
-#endif
-#endif
-
-  if (n_drives < 1) {
+  if (n_burners < 1) {
     GtkMessageDialog *dialog = (GtkMessageDialog *) gtk_message_dialog_new (NULL,
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING,
                                     GTK_BUTTONS_CLOSE,
-                                    ((const gchar *) _("No drives are currently available!")));
+                                    ((const gchar *) _("No burners are currently available")));
     gtk_message_dialog_format_secondary_text (dialog,
-                                    _("Maybe there is a mounted media in the drive?\n\nPlease unmount and restart the application.\n\nIf no media is inserted, check that you have r/w access to the drive with the current user."));
+                                    _("Possibly the disc(s) are in use, and cannot get accessed.\n\n"
+                                      "Please unmount and restart the application.\n\n"
+                                      "If no disc is in the drive, check that you have read and write access to the drive with the current user."));
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (GTK_WIDGET (dialog));
   }
 
+
+  /*----------Transcoder--------------------------------------------------*/
+
   if (!transcoder_selection) {
     /* select the best available */
 #ifdef HAVE_GST
@@ -334,11 +334,13 @@ main (int argc, char **argv)
              transcoder_selection);
     g_print ("\n");
     print_available_transcoders();
+    gdk_threads_leave ();
+    burn_finish ();
     return EXIT_FAILURE;
   }
 
   if (!xfburn_transcoder_is_initialized (transcoder, &error)) {
-    g_warning ("Failed to initialize %s transcoder: %s\n\t(falling back to basic implementation)", xfburn_transcoder_get_name (transcoder), error->message);
+    xfce_warn (_("Failed to initialize %s transcoder: %s\n\t(falling back to basic implementation)"), xfburn_transcoder_get_name (transcoder), error->message);
     g_error_free (error);
     g_object_unref (transcoder);
     transcoder = XFBURN_TRANSCODER (xfburn_transcoder_basic_new ());
@@ -347,7 +349,8 @@ main (int argc, char **argv)
   }
   xfburn_transcoder_set_global (transcoder);
 
-  /* evaluate parsed command line options */
+
+  /*----------evaluate parsed command line action options-------------------------*/
 
   if (show_main) {
     xfburn_main_enter_main_window ();
@@ -370,7 +373,7 @@ main (int argc, char **argv)
       if (g_file_test (image_fullname, G_FILE_TEST_EXISTS))
 	xfburn_burn_image_dialog_set_filechooser_name (dialog, image_fullname);
       else
-	xfce_err ( g_strdup_printf ( _("Image file '%s' does not exist!"), image_fullname));
+	xfce_err ( g_strdup_printf ( _("Image file '%s' does not exist."), image_fullname));
     }
 
     gtk_dialog_run (GTK_DIALOG (dialog));
@@ -384,7 +387,8 @@ main (int argc, char **argv)
   }
 
 
-  /* main window */
+  /*----------main window--------------------------------------------------*/
+
   if (!other_action || show_main) {
     xfburn_main_enter_main_window ();
     mainwin = xfburn_main_window_new ();
@@ -398,9 +402,13 @@ main (int argc, char **argv)
       xfburn_main_window_add_audio_composition_with_files (XFBURN_MAIN_WINDOW (mainwin), argc-1, argv+1);
   }
 
+
   gtk_main ();
 
-  /* shutdown */
+
+  /*----------shutdown--------------------------------------------------*/
+
+  g_object_unref (devlist);
   g_object_unref (transcoder);
 
 #ifdef HAVE_HAL
@@ -414,7 +422,7 @@ main (int argc, char **argv)
   xfburn_settings_flush ();
   xfburn_settings_free ();
   
-  xfburn_device_list_free ();
+  burn_finish ();
 
   gdk_threads_leave ();
 
